//------------------------------------------------------------------------
//Character.cpp	
//------------------------------------------------------------------------
#include "stdafx.h"
#include "VixarLibrary.h"

//------------------------------------------------------------------------
//LN^[NX̃RXgN^
//------------------------------------------------------------------------
CCharacter::CCharacter()
{
	m_animation = 0;
	m_current_animation = 0;
	m_animation_loop = true;
	m_animation_last = false;
	m_animation_playing = true;
	D3DXMatrixIdentity(&m_transform);
	m_move = D3DXVECTOR3(0,0,0);
	m_rotate = D3DXVECTOR3(0,0,0);
	m_scale = D3DXVECTOR3(1,1,1);
	m_collision_model = -1;
};

//------------------------------------------------------------------------
//LN^[NX̃RXgN^
//------------------------------------------------------------------------
CCharacter::CCharacter(std::string s)
{
	m_animation = 0;
	m_animation_names.clear();
	m_animation_names.push_back(s);
	m_current_animation = 0;
	m_animation_loop = true;
	m_animation_last = false;
	m_animation_playing = true;
	D3DXMatrixIdentity(&m_transform);
	m_move = D3DXVECTOR3(0,0,0);
	m_rotate = D3DXVECTOR3(0,0,0);
	m_scale = D3DXVECTOR3(1,1,1);
	m_collision_model = -1;
}

//------------------------------------------------------------------------
//CCharacterfXgN^
//------------------------------------------------------------------------
CCharacter::~CCharacter()
{
	//_obt@
	for( vector<CModel>::iterator model = m_models.begin();model != m_models.end();++model )
	{
		model->ReleaseVB();
	}
	//}eA
	for ( vector<CMaterial>::iterator m = m_materials.begin();m != m_materials.end(); ++m )
	{
		m->ReleaseTexture();
	}
}

//------------------------------------------------------------------------
//Zq=̃I[o[[h
//------------------------------------------------------------------------
CCharacter& CCharacter::operator=(const CCharacter& pt)
{
	m_models = pt.m_models;
	m_materials = pt.m_materials;
	m_animation_names = pt.m_animation_names;
	m_transform = pt.m_transform;
	m_animation = pt.m_animation;
	m_current_animation = pt.m_current_animation;
	m_animation_loop = pt.m_animation_loop;
	m_animation_last = pt.m_animation_last;
	m_animation_playing = pt.m_animation_playing;
	m_move = pt.m_move;
	m_rotate = pt.m_rotate;
	m_scale = pt.m_scale;
	m_collision_model = pt.m_collision_model;

	return *this;
}

//------------------------------------------------------------------------
//LN^[̃gXtH[ƕ`
//------------------------------------------------------------------------
void CCharacter::TransformAndDrawModels(LPDIRECT3DDEVICE9 dev)
{
	D3DXMATRIX matrix;

	//f
	for( int m = 0; m < (int)m_models.size(); ++m )
	{
		//蔻pf͕`悵Ȃ
		if ( m == m_collision_model )
		{
			continue;
		}
		CModel* model = &m_models[m];
		//݂̃Aj[ṼgXtH[擾
		matrix = model->GetAnimation(m_current_animation,(int)m_animation) * m_transform;
		//v~eBu`
		model->DrawPrimitive(dev,&matrix,m_materials);
	}
	//Aj[VĐȂAj[Vi߂
	if ( m_animation_playing )
	{
		m_animation++;
	}
	//Aj[VŌ܂ōĐIꍇ
	float last = (float)m_models[0].GetAnimationSize(m_current_animation);
	if ( GetModelsSize() > 0 && m_animation >= last )
	{
		//[vĐȂꍇ
		if ( !m_animation_loop )
		{
			//Đ~
			m_animation_playing = false;
		}
		//ŌŒ~邩H
		if ( m_animation_last )
		{
			//Aj[VŌŒ~
			m_animation = last - 1;
			//Đ~
			m_animation_playing = false;
		}
		else
		{
			//Aj[Vŏɖ߂
			m_animation = 0;
		}
	}
}

//------------------------------------------------------------------------
//݂̃Aj[Vw肵čĐ
//------------------------------------------------------------------------
void CCharacter::PlayAnimation(int current,bool loop,bool last)
{
	if ( current >= 0 && GetModelsSize() > 0 && current < m_models[0].GetAnimationSize() )
	{
		//݂̃Aj[VZbg
		m_current_animation = current;
	}
	else
	{
		m_current_animation = 0;
	}
	//[vĐtOZbg
	m_animation_loop = loop;
	//ŌŎ~܂邩H
	m_animation_last = last;
	//Aj[VĐ
	m_animation_playing = true;
	//Aj[Vŏ
	m_animation = 0;
}

//------------------------------------------------------------------------
//eNX`̓ǂݍ
//------------------------------------------------------------------------
void CCharacter::LoadTexture(LPDIRECT3DDEVICE9 dev)
{
	for ( vector<CMaterial>::iterator m = m_materials.begin();m != m_materials.end(); ++m )
	{
		//eNX`ǂݍ
		m->LoadTexture(dev);
	}
}

//------------------------------------------------------------------------
//_obt@쐬	
//------------------------------------------------------------------------
void CCharacter::CreateVertexBuffer(LPDIRECT3DDEVICE9 dev)
{
	for( vector<CModel>::iterator model = m_models.begin(); model != m_models.end(); ++model )
	{
		//_obt@
		model->ReleaseVB();
		//_obt@쐬
		model->CreateVertexBuffer(dev,m_materials);
	}
}

//------------------------------------------------------------------------
//LN^[Ƃ̓蔻
//posuC̈ʒuv
//diruČv
//distanceuC̈ʒu瓖ʒu܂ł̋v
//nomaluʂ̖@v
//------------------------------------------------------------------------
bool CCharacter::Collision(D3DXVECTOR3 pos,D3DXVECTOR3 dir,float* distance,D3DXVECTOR3* normal)
{
	//蔻pfꍇ
	if ( m_collision_model >= 0 && m_collision_model < (int)m_models.size()  )
	{
		if ( m_models[m_collision_model].Collision(pos,dir,distance,normal) )
		{
			return true;
		}
	}
	else for( vector<CModel>::iterator model = m_models.begin(); model != m_models.end(); ++model )
	{
		//蔻pfȂꍇ͑SẴfœ蔻𒲂ׂ
		if ( model->Collision(pos,dir,distance,normal) )
		{
			return true;
		}
	}

	return false;
}

//------------------------------------------------------------------------
//LN^[ƃ}EX̓蔻
//------------------------------------------------------------------------
bool CCharacter::MousePick(int x,int y,LPDIRECT3DDEVICE9 dev,D3DXVECTOR3* pos)
{
	//蔻pfꍇ
	if ( m_collision_model >= 0 && m_collision_model < (int)m_models.size()  )
	{
		//NCAgW(x,y)ɓf̖ʔԍ(ĂȂꍇ-1Ԃ)
		int face = m_models[m_collision_model].MousePick(x,y,dev,pos);
		if ( face >= 0 )
		{
			return true;
		}
	}
	else for( vector<CModel>::iterator model = m_models.begin(); model != m_models.end(); ++model )
	{
		//蔻pfȂꍇ͑SẴfœ蔻𒲂ׂ
		int face = model->MousePick(x,y,dev,pos);
		//NCAgW(x,y)ɓf̖ʔԍ(ĂȂꍇ-1Ԃ)
		if ( face >= 0 )
		{
			return true;
		}
	}

	return false;
}
//------------------------------------------------------------------------
//LN^[̃Rs[
//------------------------------------------------------------------------
void CCharacter::Copy(CCharacter* pt)
{
	m_models = pt->m_models;
	m_materials = pt->m_materials;
	m_animation_names = pt->m_animation_names;
}

//------------------------------------------------------------------------
// 3DW2DWɃgXtH[܂B
//------------------------------------------------------------------------
POINT CCharacter::GetScreenPosition(LPDIRECT3DDEVICE9 dev)
{
	D3DVIEWPORT9 viewPort;
	D3DXMATRIX matProj,matView;
	D3DXVECTOR4 vw;

	dev->GetViewport(&viewPort);
	dev->GetTransform(D3DTS_VIEW,&matView);
	dev->GetTransform(D3DTS_PROJECTION,&matProj);

	// [h}gNXōWϊ܂B
	D3DXVec3Transform(&vw,&D3DXVECTOR3(0,0,0),&m_transform);

	// r[}gNXōWϊ܂B
	D3DXVec4Transform(&vw,&vw,&matView);

	// vWFNV}gNXōWϊ܂B
	D3DXVec4Transform(&vw,&vw,&matProj);

	// r[|[g񂩂NCAgW擾܂B
	int width = viewPort.Width/2;
	int height = viewPort.Height/2;
	POINT p;
	p.x = (int) (vw.x / vw.w * width ) + width;
	p.y	= (int)-(vw.y / vw.w * height) + height;

	return p;
}

//------------------------------------------------------------------------
// 蔻pf(fuCollisionv)ȂT
//------------------------------------------------------------------------
void CCharacter::SearchCollisionModel()
{
	m_collision_model = -1;
	for( int m = 0; m < (int)m_models.size(); ++m )
	{
		if ( !_stricmp("collision",m_models[m].GetName().c_str()) )
		{
			m_collision_model = m;
			return;
		}
	}
}

//------------------------------------------------------------------------
//Aj[V擾
//------------------------------------------------------------------------
const char* CCharacter::GetAnimationName(int i)
{
	return m_animation_names[i].c_str();
}

//------------------------------------------------------------------------
//Aj[Vǉ
//------------------------------------------------------------------------
void CCharacter::PushBackAnimationName(std::string s)
{
	m_animation_names.push_back(s);
}

//------------------------------------------------------------------------
//Aj[V~
//------------------------------------------------------------------------
void CCharacter::StopAnimation()
{
	m_animation_playing = false;
}

//------------------------------------------------------------------------
//Aj[VĐ擾
//------------------------------------------------------------------------
bool CCharacter::GetAnimationPlaying()
{
	return m_animation_playing;
}

//------------------------------------------------------------------------
//Aj[VŌŒ~邩擾
//------------------------------------------------------------------------
bool CCharacter::GetAnimationLast()
{
	return m_animation_last;
}

//------------------------------------------------------------------------
//݂̃Aj[V擾
//------------------------------------------------------------------------
int CCharacter::GetCurrentAnimation()
{
	return m_current_animation;
}

//------------------------------------------------------------------------
//Aj[V擾
//------------------------------------------------------------------------
float CCharacter::GetAnimation()
{
	return m_animation;
}

//------------------------------------------------------------------------
//gXtH[Zbg
//------------------------------------------------------------------------
void CCharacter::SetTransform(D3DXMATRIX m)
{
	m_transform = m;
}

//------------------------------------------------------------------------
//gXtH[擾
//------------------------------------------------------------------------
D3DXMATRIX CCharacter::GetTransform()
{
	return m_transform;
}

//------------------------------------------------------------------------
//gXtH[猻݈ʒu擾
//------------------------------------------------------------------------
D3DXVECTOR3 CCharacter::GetPosition()
{
	return D3DXVECTOR3(m_transform._41,m_transform._42,m_transform._43);
}

//------------------------------------------------------------------------
//fz̍ŏ
//------------------------------------------------------------------------
vector<CModel>::iterator CCharacter::GetModelsBegin()
{
	return m_models.begin();
}

//------------------------------------------------------------------------
//fz̏I擾
//------------------------------------------------------------------------
vector<CModel>::iterator CCharacter::GetModelsEnd()
{
	return m_models.end();
}

//------------------------------------------------------------------------
//fz̃TCY擾
//------------------------------------------------------------------------
int CCharacter::GetModelsSize()
{
	return (int)m_models.size();
}

//------------------------------------------------------------------------
//fz̃TCYύX
//------------------------------------------------------------------------
void CCharacter::ResizeModels(int i)
{
	m_models.resize(i);
}

//------------------------------------------------------------------------
//fz̍Ō擾
//------------------------------------------------------------------------
CModel* CCharacter::GetModelsBack()
{
	return &m_models.back();
}

//------------------------------------------------------------------------
//f擾
//------------------------------------------------------------------------
CModel* CCharacter::GetModel(int i)
{
	return &m_models[i];
}

//------------------------------------------------------------------------
//f̃|C^擾
//------------------------------------------------------------------------
CModel CCharacter::GetModel2(int i)
{
	return m_models[i];
}

//------------------------------------------------------------------------
//fzɒǉ
//------------------------------------------------------------------------
void CCharacter::PushBackModel(CModel m)
{
	m_models.push_back(m);
}

//------------------------------------------------------------------------
//f擾
//------------------------------------------------------------------------
std::string CCharacter::GetModelName(int i)
{
	return m_models[i].GetName();
}

//------------------------------------------------------------------------
//ړZbg
//------------------------------------------------------------------------
void CCharacter::SetMove(float x,float y,float z)
{
	m_move = D3DXVECTOR3(x,y,z);
}

//------------------------------------------------------------------------
//ړZ
//------------------------------------------------------------------------
void CCharacter::AddMove(float x,float y,float z)
{
	m_move += D3DXVECTOR3(x,y,z);
}

//------------------------------------------------------------------------
//ړZ
//------------------------------------------------------------------------
D3DXVECTOR3 CCharacter::MultiplyMove(float x,float y,float z)
{
	m_move.x *= x;
	m_move.y *= y;
	m_move.z *= z;
	return m_move;
}

//------------------------------------------------------------------------
//ړ擾
//------------------------------------------------------------------------
D3DXVECTOR3 CCharacter::GetMove()
{
	return m_move;
}

//------------------------------------------------------------------------
//Xړ擾
//------------------------------------------------------------------------
float CCharacter::GetMoveX()
{
	return m_move.x;
}

//------------------------------------------------------------------------
//Yړ擾
//------------------------------------------------------------------------
float CCharacter::GetMoveY()
{
	return m_move.y;
}

//------------------------------------------------------------------------
//Zړ擾
//------------------------------------------------------------------------
float CCharacter::GetMoveZ()
{
	return m_move.z;
}

//------------------------------------------------------------------------
//]Zbg
//------------------------------------------------------------------------
void CCharacter::SetRotate(float x,float y,float z)
{
	m_rotate = D3DXVECTOR3(x,y,z);
}

//------------------------------------------------------------------------
//]Z
//------------------------------------------------------------------------
void CCharacter::AddRotate(float x,float y,float z)
{
	m_rotate += D3DXVECTOR3(x,y,z);
}

//------------------------------------------------------------------------
//]擾
//------------------------------------------------------------------------
D3DXVECTOR3 CCharacter::GetRotate()
{
	return m_rotate;
}

//------------------------------------------------------------------------
//X]擾
//------------------------------------------------------------------------
float CCharacter::GetRotateX()
{
	return m_rotate.x;
}

//------------------------------------------------------------------------
//Y]擾
//------------------------------------------------------------------------
float CCharacter::GetRotateY()
{
	return m_rotate.y;
}

//------------------------------------------------------------------------
//Z]擾
//------------------------------------------------------------------------
float CCharacter::GetRotateZ()
{
	return m_rotate.z;
}

//------------------------------------------------------------------------
//XP[Zbg
//------------------------------------------------------------------------
void CCharacter::SetScale(float x,float y,float z)
{
	m_scale = D3DXVECTOR3(x,y,z);
}

//------------------------------------------------------------------------
//XP[Z
//------------------------------------------------------------------------
void CCharacter::AddScale(float x,float y,float z)
{
	m_scale += D3DXVECTOR3(x,y,z);
}

//------------------------------------------------------------------------
//XP[擾
//------------------------------------------------------------------------
D3DXVECTOR3 CCharacter::GetScale()
{
	return m_scale;
}

//------------------------------------------------------------------------
//}eAzɒǉ
//------------------------------------------------------------------------
void CCharacter::PushBackMaterial(const CMaterial& mat)
{
	m_materials.push_back(mat);
}

//------------------------------------------------------------------------
//}eAZbg
//------------------------------------------------------------------------
void CCharacter::SetMaterials(std::vector<CMaterial> &mat)
{
	m_materials = mat;
}

//------------------------------------------------------------------------
//}eAzTCY擾
//------------------------------------------------------------------------
int CCharacter::GetMaterialSize()
{
	return (int)m_materials.size();
}

//------------------------------------------------------------------------
//}eA|C^擾
//------------------------------------------------------------------------
CMaterial* CCharacter::GetMaterial(int i)
{
	return &m_materials[i];
}

//------------------------------------------------------------------------
//Zq̃I[o[[h
//------------------------------------------------------------------------
CCharacter::CCharacter (const CCharacter& pt)
{
	*this = pt;
}
